-------------------------------------------------------------------------------
-- Company    : ope
-- Author     : Olaf Petzold
--
-- Project    : Logic Analyzer
-- Title      : PRBS 16
-- File       : LFSR16.vhd
-- Standard   : VHDL'93
-- Create Date: 2005-10-02
-- Last update: 2005-10-02
-- Platform   : generic
-- Description: 
--    This is part of the logic analyzer project. For
--    further informations see at http://www.mikrocontroller.net.
--   
--    Pattern Generator PRBS 	used for e.g. RAM Test, Probe Tests
--
-- Copyright  : Copyright (c) 2005 Olaf Petzold
-- Revision   : $Id: LFSR16.vhd,v 1.2 2005/10/02 21:28:38 olaf Exp $
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity LFSR16 is
   generic (
      RESET_ACTIVE : std_logic := '1'
   );
   port (
      reset    : in  std_logic;
      clk		: in  std_logic;
      q			: out std_logic_vector(15 downto 0)
   );
end;


architecture RTL of LFSR16 is

   signal LFSR: std_logic_vector(15 downto 0) := (others => '1');

begin

   proc: process(LFSR, clk, reset)

      variable feedback : std_logic;
      
   begin
      if (reset = RESET_ACTIVE) then
         LFSR     <= (others => '1');
      elsif rising_edge(clk) then
         feedback := LFSR(15);
         LFSR(0)  <= feedback;
         LFSR(1)  <= LFSR(0);
         LFSR(2)  <= LFSR(1) xor feedback;
         LFSR(3)  <= LFSR(2) xor feedback;
         LFSR(4)  <= LFSR(3);
         LFSR(5)  <= LFSR(4) xor feedback;
         LFSR(6)  <= LFSR(5);
         LFSR(7)  <= LFSR(6);
         LFSR(8)  <= LFSR(7);
         LFSR(9)  <= LFSR(8);
         LFSR(10) <= LFSR(9);
         LFSR(11) <= LFSR(10);
         LFSR(12) <= LFSR(11);
         LFSR(13) <= LFSR(12);
         LFSR(14) <= LFSR(13);
         LFSR(15) <= LFSR(14);
      end if;
      
      q <= LFSR;
      
   end process;
end;
